<?php

namespace Drupal\layout_paragraphs_restrictions\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\layout_paragraphs\Event\LayoutParagraphsAllowedTypesEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Provides Layout Paragraphs Restrictions.
 */
class LayoutParagraphsRestrictions implements EventSubscriberInterface {

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * An array of restriction rules.
   *
   * @var array
   */
  protected $restrictions;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructor.
   *
   * We use dependency injection get Messenger.
   *
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The messenger service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   */
  public function __construct(Messenger $messenger, ConfigFactoryInterface $config_factory, LoggerChannelFactoryInterface $logger_factory) {
    $this->messenger = $messenger;
    $this->restrictions = $config_factory
      ->get('layout_paragraphs_restrictions.settings')
      ->get('restrictions');
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritDoc}
   */
  public static function getSubscribedEvents() {
    return [
      LayoutParagraphsAllowedTypesEvent::EVENT_NAME => ['typeRestrictions', -100],
    ];
  }

  /**
   * Restricts available types based on settings in layout.
   *
   * The current context will have the following possible key/value pairs:
   *
   * - parent_uuid: The UUID of the parent component.
   * - parent_type: The bundle of the parent component.
   * - sibling_uuid: The UUID of the sibling component.
   * - sibling_type: The bundle of the sibling component.
   * - region: The region name (_root if no region is set).
   * - layout: The layout plugin ID.
   * - placement: The placement of the component (before or after).
   * - field_name: The field name.
   *
   * @param \Drupal\layout_paragraphs\Event\LayoutParagraphsAllowedTypesEvent $event
   *   The allowed types event.
   */
  public function typeRestrictions(LayoutParagraphsAllowedTypesEvent $event) {
    $context = $event->getContext();
    if ($context['parent_uuid']) {
      $parent_paragraph = $event->getLayout()->getComponentByUuid($event->getParentUuid())->getEntity();
      $section = $event->getLayout()->getLayoutSection($parent_paragraph);
      $layout_plugin_id = $section->getLayoutId();
      $context['parent_type'] = $parent_paragraph->bundle();
      $context['layout'] = $layout_plugin_id;
    }
    if ($context['sibling_uuid']) {
      $sibling_paragraph = $event->getLayout()->getComponentByUuid($context['sibling_uuid'])->getEntity();
      $context['sibling_type'] = $sibling_paragraph->bundle();
    }
    if (empty($context['region'])) {
      $context['region'] = '_root';
    }

    // @todo These contexts are not returned by Layout Paragraphs.
    $paragraphsReferenceField = $event->getLayout()->getParagraphsReferenceField();
    $parentEntity = $paragraphsReferenceField->getEntity();

    if (empty($context['field_name'])) {
      $context['field_name'] = $paragraphsReferenceField->getName();
    }
    if (empty($context['entity_type'])) {
      $context['entity_type'] = $parentEntity->getEntityTypeId();
    }
    if (empty($context['entity_bundle'])) {
      $context['entity_bundle'] = $parentEntity->bundle();
    }

    foreach ($this->restrictions as $restriction) {
      $tests = [];
      $include = [];
      $exclude = [];

      // If context is a numerically indexed array, treat each item as a separate test context.
      // Otherwise treat it as a single test context.
      if (is_array($restriction['context']) && array_keys($restriction['context']) === range(0, count($restriction['context']) - 1)) {
        $tests = array_merge($tests, $restriction['context']);
      }
      else {
        $tests[] = $restriction['context'];
      }

      foreach ($tests as $runtest) {
        $total_tests = 0;
        $matched_tests = 0;

        foreach ($runtest as $restriction_context_key => $restriction_context_value) {
          $total_tests++;

          if (str_starts_with($restriction_context_value, '!')) {
            $restriction_context_value = substr($restriction_context_value, 1);
            $test = isset($context[$restriction_context_key])
              && $context[$restriction_context_key] !== $restriction_context_value;
          }
          else {
            $test = isset($context[$restriction_context_key])
              && $context[$restriction_context_key] === $restriction_context_value;
          }

          if ($test === TRUE) {
            $matched_tests++;
          }
        }

        // If all tests matched, apply the restrictions.
        if ($matched_tests === $total_tests) {
          if (!empty($restriction['components'])) {
            $include = array_merge($include, array_fill_keys($restriction['components'], TRUE));
          }
          if (!empty($restriction['exclude_components'])) {
            $exclude = array_merge($exclude, array_fill_keys($restriction['exclude_components'], TRUE));
          }

          if ($include) {
            $event->setTypes(array_intersect_key($event->getTypes(), $include));
          }
          if ($exclude) {
            $event->setTypes(array_diff_key($event->getTypes(), $exclude));
          }
        }
      }
    }
  }

}
